@@ -365,54 +365,135 @@ function Inspector() {
 
 Inspector.prototype = {
     _init: function() {
-        let width = 150;
-        let primary = global.get_primary_monitor();
+        let container = new Shell.GenericContainer({ width: 0,
+                                                     height: 0 });
+        container.connect('allocate', Lang.bind(this, this._allocate));
+        Main.uiGroup.add_actor(container);
+
         let eventHandler = new St.BoxLayout({ name: 'LookingGlassDialog',
                                               vertical: false,
-                                              y: primary.y + Math.floor(primary.height / 2),
                                               reactive: true });
-        eventHandler.connect('notify::allocation', Lang.bind(this, function () {
-            eventHandler.x = primary.x + Math.floor((primary.width - eventHandler.width) / 2);
-        }));
-        Main.uiGroup.add_actor(eventHandler);
-        let displayText = new St.Label();
-        eventHandler.add(displayText, { expand: true });
+        this._eventHandler = eventHandler;
+        container.add_actor(eventHandler);
+        this._displayText = new St.Label();
+        eventHandler.add(this._displayText, { expand: true });
 
-        let borderPaintTarget = null;
-        let borderPaintId = null;
-        eventHandler.connect('destroy', Lang.bind(this, function() {
-            if (borderPaintTarget != null)
-                borderPaintTarget.disconnect(borderPaintId);
-        }));
-
-        eventHandler.connect('button-press-event', Lang.bind(this, function (actor, event) {
-            Clutter.ungrab_pointer(eventHandler);
-
-            let [stageX, stageY] = event.get_coords();
-            let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
-                                                       stageX,
-                                                       stageY);
-            this.emit('target', target, stageX, stageY);
-            eventHandler.destroy();
-            this.emit('closed');
-            return true;
-        }));
-
-        eventHandler.connect('motion-event', Lang.bind(this, function (actor, event) {
-            let [stageX, stageY] = event.get_coords();
-            let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
-                                                       stageX,
-                                                       stageY);
-            let position = '[inspect x: ' + stageX + ' y: ' + stageY + ']';
-            displayText.text = '';
-            displayText.text = position + ' ' + target;
-            if (borderPaintTarget != null)
-                borderPaintTarget.disconnect(borderPaintId);
-            borderPaintTarget = target;
-            borderPaintId = Shell.add_hook_paint_red_border(target);
-            return true;
-        }));
+        this._borderPaintTarget = null;
+        this._borderPaintId = null;
+        eventHandler.connect('destroy', Lang.bind(this, this._onDestroy));
+        eventHandler.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
+        eventHandler.connect('button-press-event', Lang.bind(this, this._onButtonPressEvent));
+        eventHandler.connect('scroll-event', Lang.bind(this, this._onScrollEvent));
+        eventHandler.connect('motion-event', Lang.bind(this, this._onMotionEvent));
         Clutter.grab_pointer(eventHandler);
+        Clutter.grab_keyboard(eventHandler);
+
+        // this._target is the actor currently shown by the inspector.
+        // this._pointerTarget is the actor directly under the pointer.
+        // Normally these are the same, but if you use the scroll wheel
+        // to drill down, they'll diverge until you either scroll back
+        // out, or move the pointer outside of _pointerTarget.
+        this._target = null;
+        this._pointerTarget = null;
+    },
+
+    _allocate: function(actor, box, flags) {
+        let primary = global.get_primary_monitor();
+
+        let [minWidth, minHeight, natWidth, natHeight] =
+            this._eventHandler.get_preferred_size();
+
+        let childBox = new Clutter.ActorBox();
+        childBox.x1 = primary.x + Math.floor((primary.width - natWidth) / 2);
+        childBox.x2 = childBox.x1 + natWidth;
+        childBox.y1 = primary.y + Math.floor((primary.height - natHeight) / 2);
+        childBox.y2 = childBox.y1 + natHeight;
+        this._eventHandler.allocate(childBox, flags);
+    },
+
+    _close: function() {
+        Clutter.ungrab_pointer(this._eventHandler);
+        Clutter.ungrab_keyboard(this._eventHandler);
+        this._eventHandler.destroy();
+        this.emit('closed');
+    },
+
+    _onDestroy: function() {
+        if (this._borderPaintTarget != null)
+            this._borderPaintTarget.disconnect(this._borderPaintId);
+    },
+
+    _onKeyPressEvent: function (actor, event) {
+        if (event.get_key_symbol() == Clutter.Escape)
+            this._close();
+        return true;
+    },
+
+    _onButtonPressEvent: function (actor, event) {
+        if (this._target) {
+            let [stageX, stageY] = event.get_coords();
+            this.emit('target', this._target, stageX, stageY);
+        }
+        this._close();
+        return true;
+    },
+
+    _onScrollEvent: function (actor, event) {
+        switch (event.get_scroll_direction()) {
+        case Clutter.ScrollDirection.UP:
+            // select parent
+            let parent = this._target.get_parent();
+            if (parent != null) {
+                this._target = parent;
+                this._update(event);
+            }
+            break;
+
+        case Clutter.ScrollDirection.DOWN:
+            // select child
+            if (this._target != this._pointerTarget) {
+                let child = this._pointerTarget;
+                while (child) {
+                    let parent = child.get_parent();
+                    if (parent == this._target)
+                        break;
+                    child = parent;
+                }
+                if (child) {
+                    this._target = child;
+                    this._update(event);
+                }
+            }
+            break;
+
+        default:
+            break;
+        }
+        return true;
+    },
+
+    _onMotionEvent: function (actor, event) {
+        this._update(event);
+        return true;
+    },
+
+    _update: function(event) {
+        let [stageX, stageY] = event.get_coords();
+        let target = global.stage.get_actor_at_pos(Clutter.PickMode.ALL,
+                                                   stageX,
+                                                   stageY);
+
+        if (target != this._pointerTarget)
+            this._target = target;
+        this._pointerTarget = target;
+
+        let position = '[inspect x: ' + stageX + ' y: ' + stageY + ']';
+        this._displayText.text = '';
+        this._displayText.text = position + ' ' + this._target;
+        if (this._borderPaintTarget != null)
+            this._borderPaintTarget.disconnect(this._borderPaintId);
+        this._borderPaintTarget = this._target;
+        this._borderPaintId = Shell.add_hook_paint_red_border(this._target);
     }
 };
 
@@ -597,8 +678,9 @@ LookingGlass.prototype = {
 
         let toolbar = new St.BoxLayout({ name: 'Toolbar' });
         this.actor.add_actor(toolbar);
-        let inspectIcon = St.TextureCache.get_default().load_gicon(new Gio.ThemedIcon({ name: 'gtk-color-picker' }),
-                                                                   24);
+        let inspectIcon = St.TextureCache.get_default().load_icon_name('gtk-color-picker',
+                                                                       St.IconType.SYMBOLIC,
+                                                                       24);
         toolbar.add_actor(inspectIcon);
         inspectIcon.reactive = true;
         inspectIcon.connect('button-press-event', Lang.bind(this, function () {
@@ -701,7 +783,7 @@ LookingGlass.prototype = {
         let fontName = gconf.get_string('/desktop/gnome/interface/monospace_font_name');
         // This is mishandled by the scanner - should by Pango.FontDescription_from_string(fontName);
         // https://bugzilla.gnome.org/show_bug.cgi?id=595889
-        let fontDesc = Pango.Font.description_from_string(fontName);
+        let fontDesc = Pango.font_description_from_string(fontName);
         // We ignore everything but size and style; you'd be crazy to set your system-wide
         // monospace font to be bold/oblique/etc. Could easily be added here.
         this.actor.style =
