@@ -1,7 +1,6 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Signals = imports.signals;
-const St = imports.gi.St;
 
 const RESULT_ICON_SIZE = 24;
 
@@ -10,9 +9,10 @@ const RESULT_ICON_SIZE = 24;
 // implementations.
 const MatchType = {
     NONE: 0,
-    MULTIPLE: 1,
-    PREFIX: 2,
-    SUBSTRING: 3
+    SUBSTRING: 1,
+    MULTIPLE_SUBSTRING: 2,
+    PREFIX: 3,
+    MULTIPLE_PREFIX: 4
 };
 
 function SearchResultDisplay(provider) {
@@ -40,7 +40,7 @@ SearchResultDisplay.prototype = {
      * The terms are useful for search match highlighting.
      */
     renderResults: function(results, terms) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -67,7 +67,7 @@ SearchResultDisplay.prototype = {
      * Returns: The number of actors visible.
      */
     getVisibleResultCount: function() {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -79,14 +79,14 @@ SearchResultDisplay.prototype = {
      * available.
      */
     selectIndex: function() {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
      * Activate the currently selected search result.
      */
     activateSelected: function() {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     }
 };
 
@@ -108,7 +108,7 @@ SearchProvider.prototype = {
 
     /**
      * getInitialResultSet:
-     * @terms: Array of search terms, treated as logical OR
+     * @terms: Array of search terms, treated as logical AND
      *
      * Called when the user first begins a search (most likely
      * therefore a single term of length one or two), or when
@@ -120,14 +120,15 @@ SearchProvider.prototype = {
      * item.  Ordering of returned results is up to the discretion of the provider,
      * but you should follow these heruistics:
      *
-     *  * Put items which match multiple search terms before single matches
+     *  * Put items where the term matches multiple criteria (e.g. name and
+     *    description) before single matches
      *  * Put items which match on a prefix before non-prefix substring matches
      *
      * This function should be fast; do not perform unindexed full-text searches
      * or network queries.
      */
     getInitialResultSet: function(terms) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -144,7 +145,7 @@ SearchProvider.prototype = {
      * result set, rather than possibly performing a full re-query.
      */
     getSubsearchResultSet: function(previousResults, newTerms) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -155,7 +156,7 @@ SearchProvider.prototype = {
      * properties which describe the given search result.
      */
     getResultMeta: function(id) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -194,7 +195,7 @@ SearchProvider.prototype = {
      * Called when the user chooses a given result.
      */
     activateResult: function(id) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     },
 
     /**
@@ -205,7 +206,7 @@ SearchProvider.prototype = {
      * displaying search results for that item type.
      */
     expandSearch: function(terms) {
-        throw new Error("not implemented");
+        throw new Error('Not implemented');
     }
 };
 Signals.addSignalMethods(SearchProvider.prototype);
@@ -238,7 +239,7 @@ SearchSystem.prototype = {
     },
 
     updateSearch: function(searchString) {
-        searchString = searchString.replace(/^\s+/g, "").replace(/\s+$/g, "");
+        searchString = searchString.replace(/^\s+/g, '').replace(/\s+$/g, '');
         if (searchString == '')
             return null;
 
@@ -257,16 +258,24 @@ SearchSystem.prototype = {
         if (isSubSearch) {
             for (let i = 0; i < this._previousResults.length; i++) {
                 let [provider, previousResults] = this._previousResults[i];
-                let providerResults = provider.getSubsearchResultSet(previousResults, terms);
-                if (providerResults.length > 0)
-                    results.push([provider, providerResults]);
+                try {
+                    let providerResults = provider.getSubsearchResultSet(previousResults, terms);
+                    if (providerResults.length > 0)
+                        results.push([provider, providerResults]);
+                } catch (error) {
+                    global.log ('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
+                }
             }
         } else {
             for (let i = 0; i < this._providers.length; i++) {
                 let provider = this._providers[i];
-                let providerResults = provider.getInitialResultSet(terms);
-                if (providerResults.length > 0)
-                    results.push([provider, providerResults]);
+                try {
+                    let providerResults = provider.getInitialResultSet(terms);
+                    if (providerResults.length > 0)
+                        results.push([provider, providerResults]);
+                } catch (error) {
+                    global.log ('A ' + error.name + ' has occured in ' + provider.title + ': ' + error.message);
+                }
             }
         }
 
