@@ -1,9 +1,6 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
-const GdkPixbuf = imports.gi.GdkPixbuf;
-const Gdk = imports.gi.Gdk;
-const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -28,7 +25,8 @@ FRAME_COLOR.from_pixel(0xffffffff);
 
 const SCROLL_SCALE_AMOUNT = 100 / 5;
 
-const ZOOM_OVERLAY_FADE_TIME = 0.15;
+const LIGHTBOX_FADE_TIME = 0.1;
+const CLOSE_BUTTON_FADE_TIME = 0.1;
 
 const DRAGGING_WINDOW_OPACITY = 100;
 
@@ -125,7 +123,8 @@ WindowClone.prototype = {
                            Lang.bind(this, this._onLeave));
 
         this._draggable = DND.makeDraggable(this.actor,
-                                            { dragActorMaxSize: WINDOW_DND_SIZE,
+                                            { restoreOnSuccess: true,
+                                              dragActorMaxSize: WINDOW_DND_SIZE,
                                               dragActorOpacity: DRAGGING_WINDOW_OPACITY });
         this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
         this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
@@ -216,7 +215,10 @@ WindowClone.prototype = {
         this._zooming = true;
         this.emit('zoom-start');
 
-        this._zoomLightbox = new Lightbox.Lightbox(global.stage, false);
+        if (!this._zoomLightbox)
+            this._zoomLightbox = new Lightbox.Lightbox(global.stage,
+                                                       { fadeTime: LIGHTBOX_FADE_TIME });
+        this._zoomLightbox.show();
 
         this._zoomLocalOrig  = new ScaledPoint(this.actor.x, this.actor.y, this.actor.scale_x, this.actor.scale_y);
         this._zoomGlobalOrig = new ScaledPoint();
@@ -255,7 +257,7 @@ WindowClone.prototype = {
         [this.actor.x, this.actor.y]             = this._zoomLocalOrig.getPosition();
         [this.actor.scale_x, this.actor.scale_y] = this._zoomLocalOrig.getScale();
 
-        this._zoomLightbox.destroy();
+        this._zoomLightbox.hide();
 
         this._zoomLocalPosition  = undefined;
         this._zoomLocalScale     = undefined;
@@ -263,7 +265,6 @@ WindowClone.prototype = {
         this._zoomGlobalScale    = undefined;
         this._zoomTargetPosition = undefined;
         this._zoomStep           = undefined;
-        this._zoomLightbox       = undefined;
     },
 
     _onButtonRelease : function (actor, event) {
@@ -438,7 +439,7 @@ WindowOverlay.prototype = {
         this._parentActor.raise_top();
         Tweener.addTween(this.title,
                         { opacity: 255,
-                          time: Overview.ANIMATION_TIME,
+                          time: CLOSE_BUTTON_FADE_TIME,
                           transition: 'easeOutQuad' });
     },
 
@@ -613,7 +614,7 @@ Workspace.prototype = {
 
         // Find the desktop window
         for (let i = 0; i < windows.length; i++) {
-            if (windows[i].get_window_type() == Meta.WindowType.DESKTOP) {
+            if (windows[i].meta_window.get_window_type() == Meta.WindowType.DESKTOP) {
                 this._desktop = new DesktopClone(windows[i]);
                 break;
             }
@@ -713,15 +714,17 @@ Workspace.prototype = {
      * This function also resets the highlighted window state.
      */
     setLightboxMode: function (showLightbox) {
-        if (showLightbox) {
-            this._lightbox = new Lightbox.Lightbox(this.actor, false);
-        } else {
-            this._lightbox.destroy();
-            this._lightbox = null;
-        }
-        if (this._frame) {
+        if (!this._lightbox)
+            this._lightbox = new Lightbox.Lightbox(this.actor,
+                                                   { fadeTime: LIGHTBOX_FADE_TIME });
+
+        if (showLightbox)
+            this._lightbox.show();
+        else
+            this._lightbox.hide();
+
+        if (this._frame)
             this._frame.set_opacity(showLightbox ? 150 : 255);
-        }
     },
 
     /**
@@ -731,6 +734,9 @@ Workspace.prototype = {
      * Draw the user's attention to the given window @metaWindow.
      */
     setHighlightWindow: function (metaWindow) {
+        if (!this._lightbox)
+            return;
+
         let actor;
         if (metaWindow != null) {
             let clone = this.lookupCloneForMetaWindow(metaWindow);
@@ -1580,6 +1586,15 @@ Workspace.prototype = {
     },
 
     // Draggable target interface
+    handleDragOver : function(source, actor, x, y, time) {
+        if (source instanceof WindowClone)
+            return DND.DragMotionResult.MOVE_DROP;
+        if (source.shellWorkspaceLaunch)
+            return DND.DragMotionResult.COPY_DROP;
+
+        return DND.DragMotionResult.CONTINUE;
+    },
+
     acceptDrop : function(source, actor, x, y, time) {
         if (source instanceof WindowClone) {
             let win = source.realWindow;
