@@ -65,6 +65,7 @@
 #include "st-texture-cache.h"
 #include "st-marshal.h"
 #include "st-clipboard.h"
+#include "st-private.h"
 
 #define HAS_FOCUS(actor) (clutter_actor_get_stage (actor) && clutter_stage_get_key_focus ((ClutterStage *) clutter_actor_get_stage (actor)) == actor)
 
@@ -583,7 +584,8 @@ st_entry_enter_event (ClutterActor         *actor,
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
 
-  if (priv->hint && priv->hint_visible)
+  if (clutter_actor_contains (actor, event->source)
+      && priv->hint && priv->hint_visible)
     {
       st_widget_set_hover (ST_WIDGET (actor), TRUE);
     }
@@ -595,7 +597,8 @@ static gboolean
 st_entry_leave_event (ClutterActor         *actor,
                       ClutterCrossingEvent *event)
 {
-  st_widget_set_hover (ST_WIDGET (actor), FALSE);
+  if (!clutter_actor_contains (actor, event->related))
+    st_widget_set_hover (ST_WIDGET (actor), FALSE);
 
   return CLUTTER_ACTOR_CLASS (st_entry_parent_class)->leave_event (actor, event);
 }
