@@ -15,11 +15,14 @@ static void st_theme_node_finalize           (GObject                 *object);
 static const ClutterColor BLACK_COLOR = { 0, 0, 0, 0xff };
 static const ClutterColor TRANSPARENT_COLOR = { 0, 0, 0, 0 };
 
+extern gfloat st_slow_down_factor;
+
 G_DEFINE_TYPE (StThemeNode, st_theme_node, G_TYPE_OBJECT)
 
 static void
 st_theme_node_init (StThemeNode *node)
 {
+  node->transition_duration = -1;
   _st_theme_node_init_drawing_state (node);
 }
 
@@ -96,10 +99,16 @@ st_theme_node_finalize (GObject *object)
 
   if (node->shadow)
     {
-      st_shadow_free (node->shadow);
+      st_shadow_unref (node->shadow);
       node->shadow = NULL;
     }
 
+  if (node->text_shadow)
+    {
+      st_shadow_unref (node->text_shadow);
+      node->text_shadow = NULL;
+    }
+
   if (node->background_image)
     g_free (node->background_image);
 
@@ -233,6 +242,49 @@ st_theme_node_get_pseudo_class (StThemeNode *node)
   return node->pseudo_class;
 }
 
+/**
+ * st_theme_node_equal:
+ * @node_a: first #StThemeNode
+ * @node_b: second #StThemeNode
+ *
+ * Compare two #StThemeNodes. Two nodes which compare equal will match
+ * the same CSS rules and have the same style properties. However, two
+ * nodes that have ended up with identical style properties do not
+ * necessarily compare equal.
+ * In detail, @node_a and @node_b are considered equal iff
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>they share the same #StTheme and #StThemeContext</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>they have the same parent</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>they have the same element type</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>their id, class, pseudo-class and inline-style match</para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ * Returns: %TRUE if @node_a equals @node_b
+ */
+gboolean
+st_theme_node_equal (StThemeNode *node_a, StThemeNode *node_b)
+{
+  g_return_val_if_fail (ST_IS_THEME_NODE (node_a), FALSE);
+  g_return_val_if_fail (ST_IS_THEME_NODE (node_b), FALSE);
+
+  return node_a->parent_node == node_b->parent_node &&
+         node_a->context == node_b->context &&
+         node_a->theme == node_b->theme &&
+         node_a->element_type == node_b->element_type &&
+         !g_strcmp0 (node_a->element_id, node_b->element_id) &&
+         !g_strcmp0 (node_a->element_class, node_b->element_class) &&
+         !g_strcmp0 (node_a->pseudo_class, node_b->pseudo_class) &&
+         !g_strcmp0 (node_a->inline_style, node_b->inline_style);
+}
+
 static void
 ensure_properties (StThemeNode *node)
 {
@@ -633,8 +685,19 @@ get_length_from_term (StThemeNode *node,
       return VALUE_NOT_FOUND;
 
     case NUM_GENERIC:
-      g_warning ("length values must specify a unit");
-      return VALUE_NOT_FOUND;
+      {
+        if (num->val != 0)
+          {
+            g_warning ("length values must specify a unit");
+            return VALUE_NOT_FOUND;
+          }
+        else
+          {
+            type = ABSOLUTE;
+            multiplier = 0;
+          }
+        break;
+      }
 
     case NUM_PERCENTAGE:
       g_warning ("percentage lengths not currently supported");
@@ -899,7 +962,7 @@ do_border_property (StThemeNode   *node,
 
   if (strcmp (property_name, "") == 0)
     {
-      /* Set value for width/color/node in any order */
+      /* Set value for width/color/style in any order */
       CRTerm *term;
 
       for (term = decl->value; term; term = term->next)
@@ -921,7 +984,6 @@ do_border_property (StThemeNode   *node,
                 }
               else if (strcmp (ident, "dotted") == 0 ||
                        strcmp (ident, "dashed") == 0 ||
-                       strcmp (ident, "solid") == 0 ||
                        strcmp (ident, "double") == 0 ||
                        strcmp (ident, "groove") == 0 ||
                        strcmp (ident, "ridge") == 0 ||
@@ -992,6 +1054,97 @@ do_border_property (StThemeNode   *node,
     }
 }
 
+static void
+do_outline_property (StThemeNode   *node,
+                     CRDeclaration *decl)
+{
+  const char *property_name = decl->property->stryng->str + 7; /* Skip 'outline' */
+  ClutterColor color;
+  gboolean color_set = FALSE;
+  int width;
+  gboolean width_set = FALSE;
+
+  if (strcmp (property_name, "") == 0)
+    {
+      /* Set value for width/color/style in any order */
+      CRTerm *term;
+
+      for (term = decl->value; term; term = term->next)
+        {
+          GetFromTermResult result;
+
+          if (term->type == TERM_IDENT)
+            {
+              const char *ident = term->content.str->stryng->str;
+              if (strcmp (ident, "none") == 0 || strcmp (ident, "hidden") == 0)
+                {
+                  width = 0.;
+                  continue;
+                }
+              else if (strcmp (ident, "solid") == 0)
+                {
+                  /* The only thing we support */
+                  continue;
+                }
+              else if (strcmp (ident, "dotted") == 0 ||
+                       strcmp (ident, "dashed") == 0 ||
+                       strcmp (ident, "double") == 0 ||
+                       strcmp (ident, "groove") == 0 ||
+                       strcmp (ident, "ridge") == 0 ||
+                       strcmp (ident, "inset") == 0 ||
+                       strcmp (ident, "outset") == 0)
+                {
+                  /* Treat the same as solid */
+                  continue;
+                }
+
+              /* Presumably a color, fall through */
+            }
+
+          if (term->type == TERM_NUMBER)
+            {
+              result = get_length_from_term_int (node, term, FALSE, &width);
+              if (result != VALUE_NOT_FOUND)
+                {
+                  width_set = result == VALUE_FOUND;
+                  continue;
+                }
+            }
+
+          result = get_color_from_term (node, term, &color);
+          if (result != VALUE_NOT_FOUND)
+            {
+              color_set = result == VALUE_FOUND;
+              continue;
+            }
+        }
+
+    }
+  else if (strcmp (property_name, "-color") == 0)
+    {
+      if (decl->value == NULL || decl->value->next != NULL)
+        return;
+
+      if (get_color_from_term (node, decl->value, &color) == VALUE_FOUND)
+        /* Ignore inherit */
+        color_set = TRUE;
+    }
+  else if (strcmp (property_name, "-width") == 0)
+    {
+      if (decl->value == NULL || decl->value->next != NULL)
+        return;
+
+      if (get_length_from_term_int (node, decl->value, FALSE, &width) == VALUE_FOUND)
+        /* Ignore inherit */
+        width_set = TRUE;
+    }
+
+  if (color_set)
+    node->outline_color = color;
+  if (width_set)
+    node->outline_width = width;
+}
+
 static void
 do_padding_property_term (StThemeNode *node,
                           CRTerm      *term,
@@ -1100,6 +1253,9 @@ _st_theme_node_ensure_geometry (StThemeNode *node)
       node->border_color[j] = TRANSPARENT_COLOR;
     }
 
+  node->outline_width = 0;
+  node->outline_color = TRANSPARENT_COLOR;
+
   node->width = -1;
   node->height = -1;
   node->min_width = -1;
@@ -1114,6 +1270,8 @@ _st_theme_node_ensure_geometry (StThemeNode *node)
 
       if (g_str_has_prefix (property_name, "border"))
         do_border_property (node, decl);
+      else if (g_str_has_prefix (property_name, "outline"))
+        do_outline_property (node, decl);
       else if (g_str_has_prefix (property_name, "padding"))
         do_padding_property (node, decl);
       else if (strcmp (property_name, "width") == 0)
@@ -1179,6 +1337,27 @@ st_theme_node_get_border_radius (StThemeNode *node,
   return node->border_radius[corner];
 }
 
+int
+st_theme_node_get_outline_width (StThemeNode  *node)
+{
+  g_return_val_if_fail (ST_IS_THEME_NODE (node), 0);
+
+  _st_theme_node_ensure_geometry (node);
+
+  return node->outline_width;
+}
+
+void
+st_theme_node_get_outline_color (StThemeNode  *node,
+                                 ClutterColor *color)
+{
+  g_return_if_fail (ST_IS_THEME_NODE (node));
+
+  _st_theme_node_ensure_geometry (node);
+
+  *color = node->outline_color;
+}
+
 int
 st_theme_node_get_width (StThemeNode *node)
 {
@@ -1262,6 +1441,7 @@ _st_theme_node_ensure_background (StThemeNode *node)
   node->background_computed = TRUE;
   node->background_color = TRANSPARENT_COLOR;
   node->background_gradient_type = ST_GRADIENT_NONE;
+  node->background_position_set = FALSE;
 
   ensure_properties (node);
 
@@ -1288,6 +1468,7 @@ _st_theme_node_ensure_background (StThemeNode *node)
           node->background_color = TRANSPARENT_COLOR;
           g_free (node->background_image);
           node->background_image = NULL;
+          node->background_position_set = FALSE;
 
           for (term = decl->value; term; term = term->next)
             {
@@ -1316,6 +1497,27 @@ _st_theme_node_ensure_background (StThemeNode *node)
                 }
             }
         }
+      else if (strcmp (property_name, "-position") == 0)
+        {
+          GetFromTermResult result = get_length_from_term_int (node, decl->value, FALSE, &node->background_position_x);
+          if (result == VALUE_NOT_FOUND)
+            {
+              node->background_position_set = FALSE;
+              continue;
+            }
+          else
+            node->background_position_set = TRUE;
+            
+           result = get_length_from_term_int (node, decl->value->next, FALSE, &node->background_position_y);
+
+          if (result == VALUE_NOT_FOUND)
+            {
+              node->background_position_set = FALSE;
+              continue;
+            }
+          else
+            node->background_position_set = TRUE;
+        }
       else if (strcmp (property_name, "-color") == 0)
         {
           GetFromTermResult result;
@@ -1505,6 +1707,33 @@ st_theme_node_get_padding (StThemeNode *node,
   return node->padding[side];
 }
 
+/**
+ * st_theme_node_get_transition_duration:
+ * @node: an #StThemeNode
+ *
+ * Get the value of the transition-duration property, which
+ * specifies the transition time between the previous #StThemeNode
+ * and @node.
+ *
+ * Returns: the node's transition duration in milliseconds
+ */
+int
+st_theme_node_get_transition_duration (StThemeNode *node)
+{
+  gdouble value = 0.0;
+
+  g_return_val_if_fail (ST_IS_THEME_NODE (node), 0);
+
+  if (node->transition_duration > -1)
+    return st_slow_down_factor * node->transition_duration;
+
+  st_theme_node_get_double (node, "transition-duration", FALSE, &value);
+
+  node->transition_duration = (int)value;
+
+  return st_slow_down_factor * node->transition_duration;
+}
+
 StTextDecoration
 st_theme_node_get_text_decoration (StThemeNode *node)
 {
@@ -1572,6 +1801,53 @@ st_theme_node_get_text_decoration (StThemeNode *node)
   return 0;
 }
 
+StTextAlign
+st_theme_node_get_text_align(StThemeNode *node)
+{
+  int i;
+
+  ensure_properties(node);
+
+  for (i = node->n_properties - 1; i >= 0; i--)
+    {
+      CRDeclaration *decl = node->properties[i];
+
+      if (strcmp(decl->property->stryng->str, "text-align") == 0)
+        {
+          CRTerm *term = decl->value;
+
+          if (term->type != TERM_IDENT || term->next)
+            continue;
+
+          if (strcmp(term->content.str->stryng->str, "inherit") == 0)
+            {
+              if (node->parent_node)
+                return st_theme_node_get_text_align(node->parent_node);
+              return ST_TEXT_ALIGN_LEFT;
+            }
+          else if (strcmp(term->content.str->stryng->str, "left") == 0)
+            {
+              return ST_TEXT_ALIGN_LEFT;
+            }
+          else if (strcmp(term->content.str->stryng->str, "right") == 0)
+            {
+              return ST_TEXT_ALIGN_RIGHT;
+            }
+          else if (strcmp(term->content.str->stryng->str, "center") == 0)
+            {
+              return ST_TEXT_ALIGN_CENTER;
+            }
+          else if (strcmp(term->content.str->stryng->str, "justify") == 0)
+            {
+              return ST_TEXT_ALIGN_JUSTIFY;
+            }
+        }
+    }
+  if(node->parent_node)
+    return st_theme_node_get_text_align(node->parent_node);
+  return ST_TEXT_ALIGN_LEFT;
+}
+
 static gboolean
 font_family_from_terms (CRTerm *term,
                         char  **family)
@@ -2182,6 +2458,72 @@ st_theme_node_get_vertical_padding (StThemeNode *node)
   return padding;
 }
 
+static void
+parse_shadow_property (StThemeNode   *node,
+                       CRDeclaration *decl,
+                       ClutterColor  *color,
+                       gdouble       *xoffset,
+                       gdouble       *yoffset,
+                       gdouble       *blur,
+                       gdouble       *spread)
+{
+  /* Set value for width/color/blur in any order */
+  CRTerm *term;
+  int n_offsets = 0;
+
+  /* default values */
+  color->red = 0x0; color->green = 0x0; color->blue = 0x0; color->alpha = 0xff;
+  *xoffset = 0.;
+  *yoffset = 0.;
+  *blur = 0.;
+  *spread = 0.;
+
+  for (term = decl->value; term; term = term->next)
+    {
+      GetFromTermResult result;
+
+      if (term->type == TERM_NUMBER)
+        {
+          gdouble value;
+          gdouble multiplier;
+
+          multiplier = (term->unary_op == MINUS_UOP) ? -1. : 1.;
+          result = get_length_from_term (node, term, FALSE, &value);
+          if (result != VALUE_NOT_FOUND)
+            {
+              switch (n_offsets++)
+                {
+                case 0:
+                  *xoffset = multiplier * value;
+                  break;
+                case 1:
+                  *yoffset = multiplier * value;
+                  break;
+                case 2:
+                  if (multiplier < 0)
+                      g_warning ("Negative blur values are "
+                                 "not allowed");
+                  *blur = value;
+                  break;
+                case 3:
+                  if (multiplier < 0)
+                      g_warning ("Negative spread values are "
+                                 "not allowed");
+                  *spread = value;
+                  break;
+                }
+              continue;
+            }
+        }
+
+      result = get_color_from_term (node, term, color);
+      if (result != VALUE_NOT_FOUND)
+        {
+          continue;
+        }
+    }
+}
+
 /**
  * st_theme_node_get_shadow:
  * @node: a #StThemeNode
@@ -2210,59 +2552,15 @@ st_theme_node_get_shadow (StThemeNode *node)
 
       if (strcmp (decl->property->stryng->str, "-st-shadow") == 0)
         {
-          /* Set value for width/color/blur in any order */
-          CRTerm *term;
-          ClutterColor color = { 0x0, 0x0, 0x0, 0xff };
-          gdouble xoffset = 0.;
-          gdouble yoffset = 0.;
-          gdouble blur = 0.;
-          gdouble spread = 0.;
-          int n_offsets = 0;
+          ClutterColor color;
+          gdouble xoffset;
+          gdouble yoffset;
+          gdouble blur;
+          gdouble spread;
 
-          for (term = decl->value; term; term = term->next)
-            {
-              GetFromTermResult result;
+          parse_shadow_property (node, decl,
+                                 &color, &xoffset, &yoffset, &blur, &spread);
 
-              if (term->type == TERM_NUMBER)
-                {
-                  gdouble value;
-                  gdouble multiplier;
-
-                  multiplier = (term->unary_op == MINUS_UOP) ? -1. : 1.;
-                  result = get_length_from_term (node, term, FALSE, &value);
-                  if (result != VALUE_NOT_FOUND)
-                    {
-                      switch (n_offsets++)
-                        {
-                        case 0:
-                          xoffset = multiplier * value;
-                          break;
-                        case 1:
-                          yoffset = multiplier * value;
-                          break;
-                        case 2:
-                          if (multiplier < 0)
-                              g_warning ("Negative blur values are "
-                                         "not allowed");
-                          blur = value;
-                          break;
-                        case 3:
-                          if (multiplier < 0)
-                              g_warning ("Negative spread values are "
-                                         "not allowed");
-                          spread = value;
-                          break;
-                        }
-                      continue;
-                    }
-                }
-
-              result = get_color_from_term (node, term, &color);
-              if (result != VALUE_NOT_FOUND)
-                {
-                  continue;
-                }
-            }
           node->shadow = st_shadow_new (&color,
                                         xoffset, yoffset,
                                         blur, spread);
@@ -2273,6 +2571,62 @@ st_theme_node_get_shadow (StThemeNode *node)
   return NULL;
 }
 
+/**
+ * st_theme_node_get_text_shadow:
+ * @node: a #StThemeNode
+ *
+ * Gets the value for the text-shadow style property
+ *
+ * Return value: (transfer none): the node's text-shadow, or %NULL
+ *   if node has no text-shadow
+ */
+StShadow *
+st_theme_node_get_text_shadow (StThemeNode *node)
+{
+  StShadow *result = NULL;
+  int i;
+
+  if (node->text_shadow_computed)
+    return node->text_shadow;
+
+  ensure_properties (node);
+
+  for (i = node->n_properties - 1; i >= 0; i--)
+    {
+      CRDeclaration *decl = node->properties[i];
+
+      if (strcmp (decl->property->stryng->str, "text-shadow") == 0)
+        {
+          ClutterColor color;
+          gdouble xoffset;
+          gdouble yoffset;
+          gdouble blur;
+          gdouble spread;
+
+          parse_shadow_property (node, decl,
+                                 &color, &xoffset, &yoffset, &blur, &spread);
+
+          result = st_shadow_new (&color,
+                                  xoffset, yoffset,
+                                  blur, spread);
+
+          break;
+        }
+    }
+
+  if (!result && node->parent_node)
+    {
+      result = st_theme_node_get_text_shadow (node->parent_node);
+      if (result)
+        st_shadow_ref (result);
+    }
+
+  node->text_shadow = result;
+  node->text_shadow_computed = TRUE;
+
+  return result;
+}
+
 static float
 get_width_inc (StThemeNode *node)
 {
@@ -2315,8 +2669,8 @@ st_theme_node_adjust_for_height (StThemeNode  *node,
 /**
  * st_theme_node_adjust_preferred_width:
  * @node: a #StThemeNode
- * @min_width_p: (inout) (allow-none): the width to adjust
- * @for_height: (inout): the height to adjust
+ * @min_width_p: (inout) (allow-none): the minimum width to adjust
+ * @natural_width_p: (inout): the natural width to adjust
  *
  * Adjusts the minimum and natural width computed for an actor by
  * adding on the necessary space for borders and padding and taking
@@ -2362,7 +2716,7 @@ st_theme_node_adjust_preferred_width (StThemeNode  *node,
  * Adjusts a "for width" passed to clutter_actor_get_preferred_height() to
  * account for borders and padding. This is a convenience function meant
  * to be called from a get_preferred_height() method of a #ClutterActor
- * subclass. The value after adjustmnet is the width available for the actor's
+ * subclass. The value after adjustment is the width available for the actor's
  * content.
  */
 void
@@ -2382,8 +2736,8 @@ st_theme_node_adjust_for_width (StThemeNode  *node,
 /**
  * st_theme_node_adjust_preferred_height:
  * @node: a #StThemeNode
- * @min_height_p: (inout) (allow-none): the height to adjust
- * @for_height: (inout): the height to adjust
+ * @min_height_p: (inout) (allow-none): the minimum height to adjust
+ * @natural_height_p: (inout): the natural height to adjust
  *
  * Adjusts the minimum and natural height computed for an actor by
  * adding on the necessary space for borders and padding and taking
@@ -2465,6 +2819,49 @@ st_theme_node_get_content_box (StThemeNode           *node,
   content_box->y2 = (int)(0.5 + content_box->y1 + content_height);
 }
 
+/**
+ * st_theme_node_get_paint_box:
+ * @node: a #StThemeNode
+ * @allocation: the box allocated to a #ClutterActor
+ * @paint_box: computed box occupied when painting the actor
+ *
+ * Gets the box used to paint the actor, including the area occupied
+ * by properties which paint outside the actor's assigned allocation.
+ * When painting @node to an offscreen buffer, this function can be
+ * used to determine the necessary size of the buffer.
+ */
+void
+st_theme_node_get_paint_box (StThemeNode           *node,
+                             const ClutterActorBox *actor_box,
+                             ClutterActorBox       *paint_box)
+{
+  StShadow *shadow;
+  ClutterActorBox shadow_box;
+  int outline_width;
+
+  g_return_if_fail (ST_IS_THEME_NODE (node));
+  g_return_if_fail (actor_box != NULL);
+  g_return_if_fail (paint_box != NULL);
+
+  shadow = st_theme_node_get_shadow (node);
+  outline_width = st_theme_node_get_outline_width (node);
+  if (!shadow && !outline_width)
+    {
+      *paint_box = *actor_box;
+      return;
+    }
+
+  if (shadow)
+    st_shadow_get_box (shadow, actor_box, &shadow_box);
+  else
+    shadow_box = *actor_box;
+
+  paint_box->x1 = MIN (actor_box->x1 - outline_width, shadow_box.x1);
+  paint_box->x2 = MAX (actor_box->x2 + outline_width, shadow_box.x2);
+  paint_box->y1 = MIN (actor_box->y1 - outline_width, shadow_box.y1);
+  paint_box->y2 = MAX (actor_box->y2 + outline_width, shadow_box.y2);
+}
+
 
 /**
  * st_theme_node_geometry_equal:
@@ -2481,6 +2878,9 @@ st_theme_node_geometry_equal (StThemeNode *node,
 {
   StSide side;
 
+  g_return_val_if_fail (ST_IS_THEME_NODE (node), FALSE);
+  g_return_val_if_fail (ST_IS_THEME_NODE (other), FALSE);
+
   _st_theme_node_ensure_geometry (node);
   _st_theme_node_ensure_geometry (other);
 
@@ -2501,3 +2901,86 @@ st_theme_node_geometry_equal (StThemeNode *node,
 
   return TRUE;
 }
+
+/**
+ * st_theme_node_paint_equal:
+ * @node: a #StThemeNode
+ * @other: a different #StThemeNode
+ *
+ * Check if st_theme_node_paint() will paint identically for @node as it does
+ * for @other. Note that in some cases this function may return %TRUE even
+ * if there is no visible difference in the painting.
+ *
+ * Return value: %TRUE if the two theme nodes paint identically. %FALSE if the
+ *   two nodes potentially paint differently.
+ */
+gboolean
+st_theme_node_paint_equal (StThemeNode *node,
+                           StThemeNode *other)
+{
+  StBorderImage *border_image, *other_border_image;
+  StShadow *shadow, *other_shadow;
+  int i;
+
+  g_return_val_if_fail (ST_IS_THEME_NODE (node), FALSE);
+  g_return_val_if_fail (ST_IS_THEME_NODE (other), FALSE);
+
+  _st_theme_node_ensure_background (node);
+  _st_theme_node_ensure_background (other);
+
+  if (!clutter_color_equal (&node->background_color, &other->background_color))
+    return FALSE;
+
+  if (node->background_gradient_type != other->background_gradient_type)
+    return FALSE;
+
+  if (node->background_gradient_type != ST_GRADIENT_NONE &&
+      !clutter_color_equal (&node->background_gradient_end, &other->background_gradient_end))
+    return FALSE;
+
+  if (g_strcmp0 (node->background_image, other->background_image) != 0)
+    return FALSE;
+
+  _st_theme_node_ensure_geometry (node);
+  _st_theme_node_ensure_geometry (other);
+
+  for (i = 0; i < 4; i++)
+    {
+      if (node->border_width[i] != other->border_width[i])
+        return FALSE;
+
+      if (node->border_width[i] > 0 &&
+          !clutter_color_equal (&node->border_color[i], &other->border_color[i]))
+        return FALSE;
+
+      if (node->border_radius[i] != other->border_radius[i])
+        return FALSE;
+    }
+
+  if (node->outline_width != other->outline_width)
+    return FALSE;
+
+  if (node->outline_width > 0 &&
+      !clutter_color_equal (&node->outline_color, &other->outline_color))
+    return FALSE;
+
+  border_image = st_theme_node_get_border_image (node);
+  other_border_image = st_theme_node_get_border_image (other);
+
+  if ((border_image == NULL) != (other_border_image == NULL))
+    return FALSE;
+
+  if (border_image != NULL && !st_border_image_equal (border_image, other_border_image))
+    return FALSE;
+
+  shadow = st_theme_node_get_shadow (node);
+  other_shadow = st_theme_node_get_shadow (other);
+
+  if ((shadow == NULL) != (other_shadow == NULL))
+    return FALSE;
+
+  if (shadow != NULL && !st_shadow_equal (shadow, other_shadow))
+    return FALSE;
+
+  return TRUE;
+}
