@@ -36,11 +36,13 @@
 
 #include "st-widget.h"
 
+#include "st-label.h"
 #include "st-marshal.h"
 #include "st-private.h"
 #include "st-texture-cache.h"
 #include "st-theme-context.h"
 #include "st-tooltip.h"
+#include "st-theme-node-transition.h"
 
 /*
  * Forward declaration for sake of StWidgetChild
@@ -53,19 +55,7 @@ struct _StWidgetPrivate
   gchar        *style_class;
   gchar        *inline_style;
 
-  ClutterActor *border_image;
-  ClutterActor *background_image;
-  ClutterColor  bg_color;
-
-  guint         border_width;
-  ClutterColor  border_color;
-
-  StGradientType bg_gradient_type;
-  ClutterColor  bg_gradient_end;
-
-  ClutterActorBox background_allocation;
-  gdouble       shadow_xoffset;
-  gdouble       shadow_yoffset;
+  StThemeNodeTransition *transition_animation;
 
   gboolean      is_stylable : 1;
   gboolean      has_tooltip : 1;
@@ -115,6 +105,8 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
+gfloat st_slow_down_factor = 1.0;
+
 G_DEFINE_ABSTRACT_TYPE (StWidget, st_widget, CLUTTER_TYPE_ACTOR);
 
 #define ST_WIDGET_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ST_TYPE_WIDGET, StWidgetPrivate))
@@ -231,6 +223,17 @@ st_widget_get_property (GObject    *gobject,
     }
 }
 
+static void
+st_widget_remove_transition (StWidget *widget)
+{
+  if (widget->priv->transition_animation)
+    {
+      g_object_run_dispose (G_OBJECT (widget->priv->transition_animation));
+      g_object_unref (widget->priv->transition_animation);
+      widget->priv->transition_animation = NULL;
+    }
+}
+
 static void
 st_widget_dispose (GObject *gobject)
 {
@@ -243,12 +246,6 @@ st_widget_dispose (GObject *gobject)
       priv->theme = NULL;
     }
 
-  if (priv->border_image)
-    {
-      clutter_actor_unparent (priv->border_image);
-      priv->border_image = NULL;
-    }
-
   if (priv->theme_node)
     {
       g_object_run_dispose (G_OBJECT (priv->theme_node));
@@ -256,6 +253,8 @@ st_widget_dispose (GObject *gobject)
       priv->theme_node = NULL;
     }
 
+  st_widget_remove_transition (actor);
+
   if (priv->tooltip)
     {
       ClutterContainer *parent;
@@ -369,12 +368,20 @@ st_widget_paint (ClutterActor *actor)
   StWidget *self = ST_WIDGET (actor);
   StThemeNode *theme_node;
   ClutterActorBox allocation;
+  guint8 opacity;
 
   theme_node = st_widget_get_theme_node (self);
 
   clutter_actor_get_allocation_box (actor, &allocation);
 
-  st_theme_node_paint (theme_node, &allocation, clutter_actor_get_paint_opacity (actor));
+  opacity = clutter_actor_get_paint_opacity (actor);
+
+  if (self->priv->transition_animation)
+    st_theme_node_transition_paint (self->priv->transition_animation,
+                                    &allocation,
+                                    opacity);
+  else
+    st_theme_node_paint (theme_node, &allocation, opacity);
 }
 
 static void
@@ -404,12 +411,6 @@ st_widget_map (ClutterActor *actor)
 
   st_widget_ensure_style ((StWidget*) actor);
 
-  if (priv->border_image)
-    clutter_actor_map (priv->border_image);
-
-  if (priv->background_image)
-    clutter_actor_map (priv->background_image);
-
   if (priv->tooltip)
     clutter_actor_map ((ClutterActor *) priv->tooltip);
 }
@@ -421,12 +422,6 @@ st_widget_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (st_widget_parent_class)->unmap (actor);
 
-  if (priv->border_image)
-    clutter_actor_unmap (priv->border_image);
-
-  if (priv->background_image)
-    clutter_actor_unmap (priv->background_image);
-
   if (priv->tooltip)
     clutter_actor_unmap ((ClutterActor *) priv->tooltip);
 }
@@ -565,15 +560,6 @@ st_widget_get_theme_node (StWidget *widget)
   return priv->theme_node;
 }
 
-static gboolean
-actor_contains (ClutterActor *widget,
-                ClutterActor *other)
-{
-  while (other != NULL && other != widget)
-    other = clutter_actor_get_parent (other);
-  return other != NULL;
-}
-
 static gboolean
 st_widget_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
@@ -582,7 +568,7 @@ st_widget_enter (ClutterActor         *actor,
 
   if (priv->track_hover)
     {
-      if (actor_contains (actor, event->source))
+      if (clutter_actor_contains (actor, event->source))
         st_widget_set_hover (ST_WIDGET (actor), TRUE);
       else
         {
@@ -608,7 +594,7 @@ st_widget_leave (ClutterActor         *actor,
 
   if (priv->track_hover)
     {
-      if (!actor_contains (actor, event->related))
+      if (!clutter_actor_contains (actor, event->related))
         st_widget_set_hover (ST_WIDGET (actor), FALSE);
     }
 
@@ -1158,7 +1144,7 @@ st_widget_remove_style_pseudo_class (StWidget    *actor,
 /**
  * st_widget_set_style:
  * @actor: a #StWidget
- * @style_class: (allow-none): a inline style string, or %NULL
+ * @style: (allow-none): a inline style string, or %NULL
  *
  * Set the inline style string for this widget. The inline style string is an
  * optional ';'-separated list of CSS properties that override the style as
@@ -1166,7 +1152,7 @@ st_widget_remove_style_pseudo_class (StWidget    *actor,
  */
 void
 st_widget_set_style (StWidget  *actor,
-                       const gchar *style)
+                     const gchar *style)
 {
   StWidgetPrivate *priv = actor->priv;
 
@@ -1217,21 +1203,71 @@ st_widget_init (StWidget *actor)
 
   actor->priv = priv = ST_WIDGET_GET_PRIVATE (actor);
   priv->is_stylable = TRUE;
+  priv->transition_animation = NULL;
 
   /* connect style changed */
   g_signal_connect (actor, "notify::name", G_CALLBACK (st_widget_name_notify), NULL);
 }
 
+static void
+on_transition_completed (StThemeNodeTransition *transition,
+                         StWidget              *widget)
+{
+  st_widget_remove_transition (widget);
+}
+
 static void
 st_widget_recompute_style (StWidget    *widget,
                            StThemeNode *old_theme_node)
 {
   StThemeNode *new_theme_node = st_widget_get_theme_node (widget);
+  int transition_duration;
+  gboolean paint_equal;
 
   if (!old_theme_node ||
       !st_theme_node_geometry_equal (old_theme_node, new_theme_node))
     clutter_actor_queue_relayout ((ClutterActor *) widget);
 
+  transition_duration = st_theme_node_get_transition_duration (new_theme_node);
+
+  paint_equal = old_theme_node && st_theme_node_paint_equal (old_theme_node, new_theme_node);
+
+  if (paint_equal)
+    st_theme_node_copy_cached_paint_state (new_theme_node, old_theme_node);
+
+  if (transition_duration > 0)
+    {
+      if (widget->priv->transition_animation != NULL)
+        {
+          st_theme_node_transition_update (widget->priv->transition_animation,
+                                           new_theme_node);
+        }
+      else if (old_theme_node && !paint_equal)
+        {
+          /* Since our transitions are only of the painting done by StThemeNode, we
+           * only want to start a transition when what is painted changes; if
+           * other visual aspects like the foreground color of a label change,
+           * we can't animate that anyways.
+           */
+
+          widget->priv->transition_animation =
+            st_theme_node_transition_new (old_theme_node,
+                                          new_theme_node,
+                                          transition_duration);
+
+          g_signal_connect (widget->priv->transition_animation, "completed",
+                            G_CALLBACK (on_transition_completed), widget);
+          g_signal_connect_swapped (widget->priv->transition_animation,
+                                    "new-frame",
+                                    G_CALLBACK (clutter_actor_queue_redraw),
+                                    widget);
+        }
+    }
+  else if (widget->priv->transition_animation)
+    {
+      st_widget_remove_transition (widget);
+    }
+
   g_signal_emit (widget, signals[STYLE_CHANGED], 0);
   widget->priv->is_style_dirty = FALSE;
 }
@@ -1559,17 +1595,16 @@ st_widget_sync_hover (StWidget *widget)
 {
   ClutterDeviceManager *device_manager;
   ClutterInputDevice *pointer;
-  ClutterActor *actor;
+  ClutterActor *pointer_actor;
 
   device_manager = clutter_device_manager_get_default ();
   pointer = clutter_device_manager_get_core_device (device_manager,
                                                     CLUTTER_POINTER_DEVICE);
-  actor = clutter_input_device_get_pointer_actor (pointer);
-
-  while (actor && actor != (ClutterActor *)widget)
-    actor = clutter_actor_get_parent (actor);
-
-  st_widget_set_hover (widget, actor == (ClutterActor *)widget);
+  pointer_actor = clutter_input_device_get_pointer_actor (pointer);
+  if (pointer_actor)
+    st_widget_set_hover (widget, clutter_actor_contains (CLUTTER_ACTOR (widget), pointer_actor));
+  else
+    st_widget_set_hover (widget, FALSE);
 }
 
 /**
@@ -1588,3 +1623,125 @@ st_widget_get_hover (StWidget *widget)
 
   return widget->priv->hover;
 }
+
+static gboolean
+append_actor_text (GString      *desc,
+                   ClutterActor *actor)
+{
+  if (CLUTTER_IS_TEXT (actor))
+    {
+      g_string_append_printf (desc, " (\"%s\")",
+                              clutter_text_get_text (CLUTTER_TEXT (actor)));
+      return TRUE;
+    }
+  else if (ST_IS_LABEL (actor))
+    {
+      g_string_append_printf (desc, " (\"%s\")",
+                              st_label_get_text (ST_LABEL (actor)));
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
+/**
+ * st_describe_actor:
+ * @actor: a #ClutterActor
+ *
+ * Creates a string describing @actor, for use in debugging. This
+ * includes the class name and actor name (if any), plus if @actor
+ * is an #StWidget, its style class and pseudo class names.
+ *
+ * Return value: the debug name.
+ */
+char *
+st_describe_actor (ClutterActor *actor)
+{
+  GString *desc;
+  const char *name;
+  int i;
+
+  if (!actor)
+    return g_strdup ("[null]");
+
+  desc = g_string_new (NULL);
+  g_string_append_printf (desc, "[%p %s", actor,
+                          G_OBJECT_TYPE_NAME (actor));
+
+  if (ST_IS_WIDGET (actor))
+    {
+      const char *style_class = st_widget_get_style_class_name (ST_WIDGET (actor));
+      const char *pseudo_class = st_widget_get_style_pseudo_class (ST_WIDGET (actor));
+      char **classes;
+
+      if (style_class)
+        {
+          classes = g_strsplit (style_class, ",", -1);
+          for (i = 0; classes[i]; i++)
+            {
+              g_strchug (classes[i]);
+              g_string_append_printf (desc, ".%s", classes[i]);
+            }
+          g_strfreev (classes);
+        }
+
+      if (pseudo_class)
+        {
+          classes = g_strsplit (pseudo_class, ",", -1);
+          for (i = 0; classes[i]; i++)
+            {
+              g_strchug (classes[i]);
+              g_string_append_printf (desc, ":%s", classes[i]);
+            }
+          g_strfreev (classes);
+        }
+    }
+
+  name = clutter_actor_get_name (actor);
+  if (name)
+    g_string_append_printf (desc, " \"%s\"", name);
+
+  if (!append_actor_text (desc, actor) && CLUTTER_IS_CONTAINER (actor))
+    {
+      GList *children, *l;
+
+      /* Do a limited search of @actor's children looking for a label */
+      children = clutter_container_get_children (CLUTTER_CONTAINER (actor));
+      for (l = children, i = 0; l && i < 20; l = l->next, i++)
+        {
+          if (append_actor_text (desc, l->data))
+            break;
+          else if (CLUTTER_IS_CONTAINER (l->data))
+            children = g_list_concat (children, clutter_container_get_children (l->data));
+        }
+      g_list_free (children);
+    }
+
+  g_string_append_c (desc, ']');
+
+  return g_string_free (desc, FALSE);
+}
+
+/**
+ * st_set_slow_down_factor:
+ *
+ * @factor: new slow-down factor
+ *
+ * Set a global factor applied to all animation durations
+ */
+void
+st_set_slow_down_factor (gfloat factor)
+{
+  st_slow_down_factor = factor;
+}
+
+/**
+ * st_get_slow_down_factor:
+ *
+ * Returns: the global factor applied to all animation durations
+ */
+gfloat
+st_get_slow_down_factor ()
+{
+  return st_slow_down_factor;
+}
